#!/usr/bin/perl 

$| = 1;  # force immediate output

&process_options;

# Find the subchips
if (@ARGV) {  # given on command line
  @subchips = @ARGV;
} else {
  @passdirs = <pass*_*>;
  %temphash = ();
  foreach $passdir (@passdirs) {
    if (-d $passdir) {
      chomp($passdir);
      $passdir =~ s/^pass[\d]+_//;
      $temphash{$passdir} = 1;
    }
  }
  foreach $mod (keys %temphash) {
    push(@subchips,$mod);
  }
}
print "subchips is @subchips\n" if ($verbose);

# Create header
$line = &addpartition("","");
foreach $pass (@passes) {
  $line = sprintf(&addfields($line,"$pass"));
}
print "$line\n"; $line = "";
$line = &addpartition("","Partition");
foreach $pass (@passes) {
  $line = sprintf(&addfields($line,"hours","Gbytes","Kgate","slack"));
}
$linelength = length($line);
print "$line\n"; $line = "";
print "=" x $linelength . "\n";

# Report on each subchip
SUBCHIP: foreach $subchip (sort @subchips) {

  # First the acs run itself
  $line = &addpartition("","${subchip} (acs)");
  $lastpass_synlogfile = "";
  PASS: foreach $pass (@passes) {
    $synlogfile = "${logdir}/${pass}_${subchip}.synlog";
    #print(-M $lastpass_synlogfile,-M $synlogfile,"\n");
    # skip if this synlog doesn't exist or is older than the last
    if ((! -r $synlogfile) ||
      ($lastpass_synlogfile && (-M $lastpass_synlogfile < -M $synlogfile))) {
      ($runtime,$memory,$area,$slack) = ("", "", "", "");
      $line = &addfields($line,$runtime,$memory,$area,$slack);
    } else {
      $lastpass_synlogfile = $synlogfile;
      ($runtime,$memory,$area,$slack) = &parse_synlog($synlogfile,"DC");
      $slack = "N/A";
      $area = "N/A";
      $line = &addfields($line,$runtime,$memory,$area,$slack);
    }
  }
  print "$line\n"; $line = "";

  # Now the make results
  # Find the existing db's to identify all partitions
  @partitions = ();
  %temphash = ();
  @dbfiles = <pass*_${subchip}/db/post_compile/*.db>;
  foreach $dbfile (@dbfiles) {
    unless ($dbfile =~ /\.h\.db/) {  # .h.db files don't count
      chomp($dbfile);
      ($mod) = ($dbfile =~ m:${pass}_${subchip}/db/post_compile/(\S+)\.db$:);
      # put in hash to uniquify
      $temphash{$mod} = 1;
    }
  }
  foreach $mod (keys %temphash) {
    #push(@{ $partitions{$subchip} },$mod);
    # make sure top level is last by using push for it and unshift for others
    if ($mod eq $subchip) {
      push(@partitions,$mod);
    } else {
      unshift(@partitions,$mod);
    }

  }
  print "partitions is @partitions\n" if ($verbose);

  foreach $partition (@partitions) {
    $line = &addpartition("","  /${partition}");
    $lastpass_synlogfile = "";
    PASS: foreach $pass (@passes) {
      $synlogfile = "${pass}_${subchip}/logs/${partition}.${synlog}";
      print "synlogfile is $synlogfile\n" if ($verbose);
    # skip if this synlog doesn't exist or is older than the last
      if ((! -r $synlogfile) ||
        ($lastpass_synlogfile && (-M $lastpass_synlogfile < -M $synlogfile))) {
        ($runtime,$memory,$area,$slack) = ("", "", "", "");
        $line = &addfields($line,$runtime,$memory,$area,$slack);
      } else {
        $lastpass_synlogfile = $synlogfile;
        ($runtime,$memory,$area,$slack) = &parse_synlog($synlogfile,"DC");
        $line = &addfields($line,$runtime,$memory,$area,$slack);
      }
    }
    print "$line\n"; $line = "";
  }

  if (-f "./${subchip}_top.synlog" ) {
    ($runtime,$memory,$area,$slack) = &parse_synlog("./${subchip}_top.synlog","run_acs.pl");
    $line = &addpartition("","  (wall clock)");
    $line = &addfields($line,$runtime);
    print "$line\n"; $line = "";
  }

  print "-" x $linelength . "\n";
}


sub parse_synlog {
  my ($file,$tool) = @_;
  my ($starttime,$endtime,$runtime) = (0,0,0);
  my ($area);
  my ($slack) = 1000000; # some big initial value
  my ($memory) = 0;

  open(SYNLOG,"< $file") || die "Cannot open synlog file $file\n";
  while(<SYNLOG>) {
    # startime
    if (/^-> Starting $tool.*\(abs:(\d+)\)/) {
      $starttime = $1;
    }
    elsif (/^-> Exiting $tool.*\(abs:(\d+)\)/) {
      $endtime = $1;
    }
    elsif (/^Total cell area:\s*(\d+)/) {
      $area = sprintf("%2.0f",$1/1000);  # convert to kgates
    }
    elsif (/^Peak memory usage:\s*(\d+)/) {
      $memory = sprintf("%2.1f",$1/1000000);  # convert to gigabytes
    }
    elsif (/^\s*slack\s+\((?:MET|VIOLATED).*([-\.\d]+)/) {
      if ($1 < $slack) {$slack = $1;}
    }
  }
  close(SYNLOG);
  # calc runtime in hours
  if ($endtime && $starttime) { # make sure numbers are valid first
    $runtime = sprintf("%2.1f",($endtime - $starttime) / 3600); 
  } else {
    $runtime = "N/A";
  }
  if ($slack < 1000000) { # make sure it is valid
    $slack = sprintf("%2.2f",$slack);
  } else {
    $slack = "N/A";
  }

  return($runtime,$memory,$area,$slack);
}

sub addfields {
  return sprintf("%s|| %-6s %-6s %-6s %-6s",@_);
}

sub addpartition {
  return sprintf("%s%-20s",@_);
}

sub process_options {

  use Getopt::Long;

  # Default non-null options
  $synlog = "synlog";
  $logdir = ".";
  # Process options
  die unless GetOptions(
    "help" => \$help,
    "verbose" => \$verbose,
    "pass=s" => \@passes,  # pass(es) to run
    "logdir=s" => \$logdir,
    );

  if ($help) {
    &show_help;
    exit;
  }

  @passes = ("pass0", "pass1", "pass2") unless (@passes);
  #@passes = ("pass0", "pass1") unless (@passes); # temp for debug

}

sub show_help {
 print STDERR "Syntax:
  $me [options] *.v

  options:
  -help                 Show this help
  -verbose              Be Chatty
  -pass <passname>      Add \"passname\" to the list of passes to run.
                        (can be used multiple times)
                        If no -pass options given, will default to run pass0,
                        pass1, and pass2.
  -logdir dir           Directory for top level log files (acs log files).
  \n";
}

# when doing require or use we must return 1
1
