#!/usr/bin/perl 
#!/usr/local/bin/perl 

$| = 1;  # force immediate output

# Make sure calling location is in INC path
use File::Basename;
$my_dir = &dirname($0);
$me = &basename($0);
unshift(@INC,"$my_dir");

# Also add it to $PATH
$ENV{"PATH"} = $ENV{"PATH"} . ":$my_dir";

&process_options;

#$newverilog = `find $vlog -name '*.v' -newer $topflatmake`;
if ((-f $topmake) && (-f $topflatmake)) {
  foreach $globalsfile (@globalsfiles) {
    $newverilog .= `find $globalsfile -newer $topflatmake`;
  }
  foreach $vlog (@vlogs) {
    $newverilog .= `find $vlog -name '*.v' -newer $topflatmake`;
  }
  if ($newverilog) {
    print "Running $mkmakefiles because these files are newer than $topflatmake:\n $newverilog\n";
    &run_mkmakefiles;
  }
} else {
  print "Running $mkmakefiles because $topmake or $topflatmake doesn't exist\n" ;
  &run_mkmakefiles;
}
  
print "\n\nRunning top-level make\n";
system "gmake --jobs --makefile $topflatmake";

sub run_mkmakefiles {

  $cmd = "$mkmakefiles";
  $cmd .= " -top $top" if ($top); # Add in top
  foreach $globalsfile (@globalsfiles) {
    $cmd .= "  -globalsfile $globalsfile";
  }
  foreach $vlog (@vlogs) {
    $cmd .= "  -vlog $vlog";
  }
  foreach $mod (@mods) {
    $cmd .= "  -mod $mod";
  }
  $cmd .= " -lsf" if ($lsf);
  $cmd .= " -recurse" if ($recurse);
  $cmd .= " -scriptdir $scriptdir" if ($scriptdir);
  $cmd .= " -logdir $logdir" if ($logdir);
  print "mk_mkfiles cmd is:\n$cmd\n" if ($verbose);
  system "$cmd";

}


sub process_options {
  use Getopt::Long;
  # Default non-null options
  $topmake = "top.make";
  $topflatmake = "top.flat.make";
  $mkmakefiles = "mk_makefiles.pl";

  # Process options
  die unless GetOptions(
    "help" => \$help,
    "verbose" => \$verbose,
    "vlog=s" => \@vlogs,
    "globalsfile=s" => \@globalsfiles,
    "mod=s" => \@mods,
    "top=s" => \$top,
    "lsf" => \$lsf,
    "recurse" => \$recurse,
    "scriptdir=s" => \$scriptdir,
    "logdir=s" => \$logdir,
    );
  
  if ($help) {
    &show_help;
    exit;
  }
  if (!@vlogs) {
    @vlogs = (".");
  }
}

sub show_help {
 print STDERR "Syntax:
  $me [options] *.v

  options:
  -help                 Show this help
  -verbose              Be Chatty
  -debug                Turn on debug messages
  -globalsfile file     \"file\" is a globals file
  -top topmodule_name   Specify top level (will be inferred if this option is
                        not used)
  -mod module_name      Specify subchip level (will be inferred if this option 
                        is not used)
  -lsf                  Submit using bsub (options are controlled in top.info.pl)
  -vlog dir             Look in \"dir\" for verilog files
                        (can be used multiple times, or used with -recurse)
  -recurse              Look for verilog files recursively in the vlog dir
  -scriptdir dir        Specify directory for scripts (will be passed to mk_makefiles.pl)
  -logdir dir           Specify directory for synlog files
  \n";
}

# when doing require or use we must return 1
1
