#!/usr/bin/perl 

$| = 1;  # force immediate output

# Make sure calling location is in INC path
use File::Basename;
$my_dir = &dirname($0);
$me = &basename($0);
unshift(@INC,"$my_dir");

&process_options;

# Call parsemake to parse makefile
require "parsemake.pm";
#&parsemake::debugon();
$pmdb = &parsemake::parsemake();

# try to figure out top if not specified
if (!$top) {
  @tops = (&parsemake::get_target_deps($pmdb,"all"));
  $top = pop(@tops);
  if (@tops) {
    die "Error: No -tops specified, and makefile appears to have more than one top mode: $top @tops\n";
  }
  $top =~ s/\.db//;
  print STDERR "No -top specified - setting top to \"$top\"\n";
}
print STDERR "top is $top\n" if ($debug);

# try to figure out mods if not specified
if (!@mods) {
  foreach $dep (&parsemake::get_target_deps($pmdb,"${top}.db")) {
    if ($dep =~ s/\.db//) {  # if it's a db dependency
      $temphash{$dep} = 1;   # uniquify using hash
    }
  }
  @mods = sort keys %temphash;
  print STDERR "No -mods specified - setting mods to \"@mods\"\n";
}

print "all: \\\n";
print "\t${top}.db \\\n" if ($top);
foreach $mod (@mods) {
  print "\t${mod}.db \\\n";
}
print "\n\n";

if ($top) {
  print "${top}.db: \\\n";
  # Add verilog source file
  foreach $dep (&parsemake::get_target_deps($pmdb,"${top}.db")) {
    print STDERR "top level vlog dep is $dep\n" if ($debug);
    print "\t$dep \\\n";
  }
  # Add db's of the top-level submodules
  #foreach $mod (@mods) {
  #  print "\t${mod}.db \\\n";
  #}
  # Add any default dependencies
  foreach $default_dep (@default_deps) {
    eval("print \"\t$default_dep \\\\\n\"");
  }

  print "\n";
  
  # now print out the command to run.
  eval("print \"\t$topcmd\n\"");
  print "\n";
}


foreach $mod (@mods) {
  print "${mod}.db: \\\n";
  
  foreach $flatdep (sort (&parsemake::get_target_deps_flat($pmdb,"${mod}.db"))) {
    print STDERR "flatdep is $flatdep\n" if ($debug);
    unless ($nodb && $flatdep =~ /\.db/) {
      print "\t$flatdep \\\n";
    }
  }
  # Add any default dependencies
  foreach $default_dep (@default_deps) {
    eval("print \"\t$default_dep \\\\\n\"");
  }

  print "\n";
  
  # now print out the command to run.
  eval("print \"\t$dcsh\n\"");
  print "\n";
}

# Try to solve the annoying gmake/RCS problem
print "
# These lines attempt to force gmake to not mess about with RCS...
%.${scr}: ;
%.v: ;

";

sub process_options {
  use Getopt::Long;

  # Default non-null options
  $makefile = "";
  $scr = "scr";
  $synlog = "synlog";
  $dcsh = "dc_shell -f \${mod}.\${scr} > \${mod}.\${synlog}";
  # Process options
  die unless GetOptions(
    "verbose" => \$verbose,
    "debug" => \$debug,
    "help" => \$help,
    "mod=s" => \@mods,
    "top=s" => \$top,
    "topcmd=s" => \$topcmd,
    "scr=s" => \$scr,
    "nodb" => \$nodb,
    "synlog=s" => \$synlog,
    "dcsh=s" => \$dcsh,
    "default_deps=s" => \@default_deps,
    );

  if ($help) {
    &show_help;
    exit;
  }

}


sub show_help {
 print STDERR "Syntax:
  $me [options] *.v

  options:
  -help                 Show this help
  -verbose              Be Chatty
  -mod \"module\"         Module to be flattened 
                        If none specified, $me will default to first level
  -nodb                 Don't include db files in the -m dependencies
  -top \"top\"          Top - this is built from the -m modules only
  -topcmd \"topcmd\"    Command to build top (syntax similar to -dcsh)
  -scr \"suffix\"       Use \"suffix\" as script suffix
                        (defaults to \"$scr\")
  -synlog \"synlog\"    Use \"synlog\" as synthesis log file suffix
                        (defaults to \"$synlog\")
  -default_deps \"dep\"  Use \"dep\" as a default dependency for all modules.
  -dcsh \"command\"     Use \"command\" as the command to build a module.  
                        This will be eval'ed by perl.  Var names available
                        include:

                        \${mod}            module being built
                        \${scr}          script suffix (from -scr or default)

                        defaults to \"$dcsh\"
  \n";
}


# when doing require or use we must return 1
1
