#!/usr/bin/perl 

$| = 1;  # force immediate output

# Make sure calling location is in INC path
use File::Basename;
$my_dir = &dirname($0);
$me = &basename($0);
unshift(@INC,"$my_dir");

&process_options;

#while (<>) {
#}

# base flatten_make command
$cmd = "
flatten_make.pl \\
  -scr dctl \\
  -synlog synlog \\
  -nodb \\
";

# Add in -top
$cmd .= " -top $top" if ($top); 

# Add in mods
foreach $mod (@mods) {
  $cmd .= "  -mod $mod";
}

# Add in the -dcsh argument
$cmd .=
"  -dcsh '\\\@echo Compiling design : \${mod} ;  run_acs.pl -top \${mod} -f top.info.pl $lsfarg $scriptdirarg $logdirarg";
# include any globals file
foreach $globalsfile (@globalsfiles) {
  $cmd .= " -globalsfile $globalsfile";
}
# complete -dcsh argument
$cmd .= " > ${logdir}\${mod}_top.\${synlog}' \\
";

# Add in the -topcmd argument
$cmd .= 
"  -topcmd '\\\@echo Compiling design : \${top} ;  run_acs.pl -top \${top} -f top.info.pl -scriptfile top.dctl -noacs $lsfarg $scriptdirarg $logdirarg";

# include any globals file
foreach $globalsfile (@globalsfiles) {
  $cmd .= " -globalsfile $globalsfile";
}
# complete -topcmd argument
$cmd .= " > ${logdir}\${top}_top.\${synlog}' \\
";


# Finish command
$cmd .= " < top.make  > top.flat.make";

# Run flatten_make.pl
#print "cmd is:\n$cmd\n";
system "$cmd";

sub process_options {
  use Getopt::Long;
  # Default non-null options
  # Process options
  die unless GetOptions(
    "help" => \$help,
    "verbose" => \$verbose,
    "globalsfiles=s" => \@globalsfiles,
    "top=s" => \$top,
    "mod=s" => \@mods,
    "lsf" => \$lsf,
    "scriptdir=s" => \$scriptdir,
    "logdir=s" => \$logdir,
    );
  
  if ($help) {
    &show_help;
    exit;
  }
  if ($lsf) {$lsfarg = "-lsf";}
  if ($scriptdir) {$scriptdirarg = "-scriptdir $scriptdir";}
  if ($logdir) {
    $logdir =~ s:/*$:/: ; # make sure it ends in /
    $logdirarg = " -logdir $logdir";
  } 
}

sub show_help {
 print STDERR "Syntax:
  $me [options] *.v

  options:
  -help                 Show this help
  -verbose              Be Chatty
  -debug                Turn on debug messages
                        (can be used multiple times)
  -globalsfile file     \"file\" is a globals file
  -top topmodule_name   Specify top level (will be inferred if this option is
                        not used)
  -mod module_name      Specify subchip level (will be inferred if this option 
                        is not used)
  -lsf                  Submit using bsub (options are controlled in top.info.pl)
  -scriptdir dir        Specify directory for scripts (will be passed to flatten_make.pl)
  -logdir dir           Specify directory for synlog files
  \n";
}

# when doing require or use we must return 1
1
