#!/usr/bin/perl 

$| = 1;  # force immediate output

# Make sure calling location is in INC path
use File::Basename;
$my_dir = &dirname($0);
$me = &basename($0);
unshift(@INC,"$my_dir");

&process_options;

#while (<>) {
#}

$cmd = "
synth_make.pl \\
  -scr dctl \\
  -synlog synlog \\
  -dcsh '\\\@echo Compiling design : \${mod}\tdc_shell -f \${mod}.\${scr} > ${logdir}\${mod}.\${synlog}' \\
  -default_dep '.synopsys_dc.setup' \\
  -default_dep 'top_env.dctl' \\
  -default_dep 'compile_env.dctl' \\
  -default_dep 'common_procs.dctl' \\
  -default_dep 'acs_common.dctl' \\
  -default_dep 'getvars.dctl' \\
  -default_dep 'compile.strategy.dctl' \\
  -default_dep 'common.const.dctl' \\
";

foreach $globalsfile (@globalsfiles) {
  $cmd .= "  -default_dep $globalsfile \\\n";
}

foreach $globalsfile (@globalsfiles) {
  $cmd .= "  $globalsfile \\\n";
}

foreach $vlog (@vlogs) {
  if ($recurse) {
    $cmd .= `find $vlog -name '*.v' -printf '\%p '`;
    $cmd .= "\\\n";
  } else {
    $cmd .= "  $vlog/*.v \\\n";
  }
}

$cmd .= "  $files > top.make";

# Run synth_make.pl
#die "cmd is:\n$cmd\n";
#print "cmd is:\n$cmd\n";
system "$cmd";

# Now make the flattened one
$cmd = "flatten_top.pl";
$cmd .= " -top $top" if ($top); # Add in top
foreach $mod (@mods) {
  $cmd .= "  -mod $mod";
}
foreach $globalsfile (@globalsfiles) {
  $cmd .= " -globalsfile $globalsfile";
}
$cmd .= " -lsf" if ($lsf);
$cmd .= " -scriptdir $scriptdir" if ($scriptdir);
$cmd .= " -logdir $logdir" if ($logdir);
#print "cmd is:\n$cmd\n";
system "$cmd";


sub process_options {
  use Getopt::Long;
  # Default non-null options
  # Process options
  die unless GetOptions(
    "help" => \$help,
    "verbose" => \$verbose,
    "vlog=s" => \@vlogs,
    "globalsfiles=s" => \@globalsfiles,
    "top=s" => \$top,
    "mod=s" => \@mods,
    "lsf" => \$lsf,
    "recurse" => \$recurse,
    "scriptdir=s" => \$scriptdir,
    "logdir=s" => \$logdir,
    );
  
  if ($help) {
    &show_help;
    exit;
  }

  if (! @vlogs) {
    push(@vlogs,".");
  }
  if ($logdir) {
    $logdir =~ s:/*$:/: ;  # make sure it ends in /
    $logdirarg = " -logdir $logdir";
  }
}

sub show_help {
 print STDERR "Syntax:
  $me [options] *.v

  options:
  -help                 Show this help
  -verbose              Be Chatty
  -debug                Turn on debug messages
  -vlog dir             Look in \"dir\" for verilog files
                        (can be used multiple times)
  -globalsfile file     \"file\" is a globals file
  -top topmodule_name   Specify top level (will be inferred if this option is
                        not used)
  -mod module_name      Specify subchip level (will be inferred if this option 
                        is not used)
  -lsf                  Submit using bsub (options are controlled in top.info.pl)
  -scriptdir dir        Specify directory for scripts (will be passed to flatten_top.pl)
  -logdir dir           Specify directory for synlog files
  \n";
}

# when doing require or use we must return 1
1
