# parsemake
# A perl package for parsing simple makefiles generated by synth_make.pl
# This will *not* parse generic, hand-written make files!

package parsemake;

my $debug = 0;

sub debugon {
  $debug = 1;
}

sub debugoff {
  $debug = 0;
}

sub parsemake {

  ($makefile) = @_;               # single argument - makefile
  my $target;
  my @deps;

  if ($makefile) {
    open(MAKEFILE,"< $makefile") || die "Cannot open makefile $makefile\n";
  } else {
    open(MAKEFILE,"< -") || die "Cannot open STDIN\n";
  }
  while (<MAKEFILE>) {
    # handle escaped newlines (see Programming Perl p122)
    chomp;
    if (s/\\$//) {
      $_ .= <MAKEFILE>;
      redo unless eof;
    }
    s/\s*;\s*//;        # get rid of trailing ; (so we don't get fooled)
    if (/^(\S+)\s*:/) { # match <target> at beginning of line
      ($target,@deps) = split(/\s+/);
      if (@deps) {      # ignore empty lists (like clean)
        $target =~ s/\s*://;  # get rid of trailing :
        print STDERR "Found target $target : @deps\n" if ($debug);
        # file it away in the "database"
        @{ $pmdb->{targetdeps}{$target}} = (@deps);
      }
    }
  }
  close(MAKEFILE);
  return $pmdb;
}

sub get_target_deps {
  my ($pmdb,$target) = @_;

  return @{ $pmdb->{targetdeps}{$target} };
}

sub get_all_targets {
  my ($pmdb) = @_;

  return keys %{$pmdb->{targetdeps}};
}

sub get_target_deps_flat {
  my ($pmdb,$target) = @_;
  my ($dep);
  my (@deps);
  my (%unique_deps);

  @deps = &get_target_deps($pmdb,$target);
  foreach $dep (&get_target_deps($pmdb,$target)) {
    push(@deps,&get_target_deps_flat($pmdb,$dep));
  }
  # use assoc array to uniquify
  foreach $dep (@deps) {
    $unique_deps{$dep} = 1;
  }
  print STDERR "Returning target $target with deps: @deps\n" if ($debug);
  return (keys %unique_deps);
}

# when doing require or use we must return 1
1

