#!/usr/bin/perl 
#
#   run_acs.pl - Run acs
#

$| = 1;  # force immediate output

# Use the better chdir
use Cwd 'chdir';

# Make sure calling location is in INC path
use File::Basename;
$my_dir = &dirname($0);
$me = &basename($0);
unshift(@INC,"$my_dir");

&process_options;

&msg("Starting $me");

# Load up the chip data (typical usage is "-f <chip>.info.pl")
foreach $file (@morefiles) {
  require $file;
}

# If there is an entry for module-specific options, parse them
if (exists $clargs{$_acs_top}) {
  print "Parsing clargs from info file $clargs{$_acs_top}\n";
  @ARGV = split(/\s+/,$clargs{$_acs_top});
  &process_options;
}

# Default options if not set by process_options or -f call
$synlog = "synlog" unless ($synlog);
$bsublog = "bsublog" unless ($bsublog);
$_acs_dctl = "dctl" unless ($_acs_dctl);
$scriptfile = "acs_common.${_acs_dctl}" unless ($scriptfile);
$joblimit = 1 unless ($joblimit);
$_acs_workdir = "work_${_acs_top}" unless ($_acs_workdir);
$_acs_top_const_file = "${_acs_top}.const.${_acs_dctl}" unless ($_acs_top_const_file);
$_acs_ungroup_cmd = "echo No ungrouping done" unless ($_acs_ungroup_cmd);
$_acs_setpartitions_cmd = "set_compile_partitions -force -all" unless ($_acs_setpartitions_cmd);
$_acs_top_envfile = "top_env.${_acs_dctl}" unless ($_acs_top_envfile);
if ($_acs_mpc_top) {@_acs_passes = (0);}  # acs mpc_top mode only works on pass0
@_acs_passes = ("0", "1", "2") unless (@_acs_passes); 
$_acs_uniquify_naming_style = "${_acs_top}_\%s_\%d" unless ($_acs_uniquify_naming_style);
$_acs_mpc_top = 0 unless ($_acs_mpc_top);
$_acs_scriptdir = "." unless ($_acs_scriptdir);
$_acs_logdir = "." unless ($_acs_logdir);
@_acs_libdirs = ("")  unless (@_acs_libdirs);

# Set misc acs vars
$_acs_home = $ENV{"PWD"};
$_acs_final_pass = @_acs_passes[$#_acs_passes];

# Parse makefile if specified.
if ($makefile) {
  require "parsemake.pm";
  $pmdb = &parsemake::parsemake($makefile);
  
  # Build a hash of the globals files so we can leave these out of the list
  foreach $globalsfile (@_acs_globals_files) {
    $globalsfiles{$globalsfile} = 1;
  }
  foreach $dep (sort (&parsemake::get_target_deps($pmdb,"${_acs_top}.db"))) {
    if (($dep =~ /\.v/) && !($globalsfiles{$dep})) {
      push(@verilog_src_files,$dep);
    }
  }
  #$ENV{"_acs_verilog_src_files"} = join(" ",@verilog_src_files);
  $_acs_verilog_src_files = join(" ",@verilog_src_files);
}

if ($noacs) {  # run_acs.pl is also abused to run the top
  @_acs_passes = ();  # block normal code by making no passes
  # Run the top-level script
  $logfile = "${_acs_logdir}/${_acs_top}.${synlog}";
  $bsublogfile = "${_acs_logdir}/${_acs_top}.${bsublog}";
  &rundc($dcshcmd,$logfile,$bsublogfile,$lsf);
}

foreach $_acs_pass (@_acs_passes) {
  print "Doing pass $_acs_pass of \{@_acs_passes\}\n";
  $_acs_passdir = "pass${_acs_pass}_${_acs_top}";
  $_acs_src = "${_acs_passdir}/db/pre_compile";
  $_acs_dest = "${_acs_passdir}/db/post_compile";

  unless ($makeonly) {
    # Set up directories
    &makedir($_acs_passdir);
  
    # Copy scripts and constraints down to ./scripts/<passdir> and 
    # ./constraints/<passdir>

    # Now do the copies
    &makedir("./scripts/$_acs_passdir");
    &copy_files("$_acs_scriptdir","./scripts/$_acs_passdir",@scripts2copy);
    &makedir("./constraints/$_acs_passdir");
    &copy_files("$_acs_scriptdir","./constraints/$_acs_passdir",@constraints2copy);
    # ACS now does a special-case on top modules - if left
    # to its own devices it will do a compile -top.  More importantly, it
    # IGNORES default.compile.  So, copy default.compile to <subchip>.compile
    # to fool it.
    system "cp -pf $_acs_scriptdir/compile.strategy.${_acs_dctl} ./scripts/$_acs_passdir/$_acs_top.compile";
    # Workaround for bug in older versions where the renaming of the
    # default strategy script has no effect.  Copy them as well.
    if (join(" ",(@scripts2copy,@constraints2copy)) =~ /compile\.strategy\.${_acs_dctl}/) {
      print "Copying compile.strategy.${_acs_dctl} to default.compile\n";
      system "cp -pf $_acs_scriptdir/compile.strategy.${_acs_dctl} ./scripts/$_acs_passdir/default.compile";
    }


  
    if ($_acs_pass eq "0") {
      # for pass 0, make the work and elab directory structure. acs_common.dctl
      # uses these to force junk files out of the root file system
      &makedir($_acs_workdir);
    }
  
    # Run the top-level script
    $logfile = "${_acs_logdir}/${_acs_passdir}.${synlog}";
    $bsublogfile = "${_acs_logdir}/${_acs_passdir}.${bsublog}";
    &rundc($dcshcmd,$logfile,$bsublogfile,$lsf);
  }

  # modify $_acs_passdir/Makefile, acs-generated scripts, etc here...
  # autoscr mods
  @autoscrfiles = <${_acs_passdir}/scripts/*.${_acs_dctl}>;
  foreach $autoscrfile (@autoscrfiles) {
    chomp($autoscrfile);
    ($origfile = $autoscrfile) =~ s/\.${_acs_dctl}/_orig.${_acs_dctl}/;
    system "mv $autoscrfile $origfile";
    open(ORIG,"< $origfile") || die "Could not open ${autoscrfile}_orig\n";
    open(NEW,"> $autoscrfile") || die "Could not open ${autoscrfile}\n";
    while (<ORIG>) {
      # comment out source of env file
      s/^(\s*source.*env)/# \1/;
      # change exit to use my &exit proc
      s/^\s*(exit|quit)/&exit/;
      print NEW;
    }
    close ORIG;
    close NEW;
  }
  # makefile mods
  # Modifying the makefile allows us to use partition-specific dc_shell
  # commands (via %partition_dcshcmds), and support lsf with partition-specific
  # bsub args (via %partition_bsubargs).
  $origmkfile = "${_acs_passdir}/Makefile_orig";
  $newmkfile = "${_acs_passdir}/Makefile";
  system "mv $newmkfile $origmkfile";
  open(ORIG,"< $origmkfile") || die "Could not open ${newmkfile}_orig\n";
  open(NEW,"> $newmkfile") || die "Could not open ${newmkfile}\n";
  while (<ORIG>) {
    # Modify lines like "$(DC_SHELL) -tcl_mode -f ...."
    if (($makefile_dcshcmd) = /^\s*\$\(DC_SHELL\)(.*)/) {
      # makefile_dcshcmd now has command less the DC_SHELL stuff at the 
      # beginning.
      # Extract partition name
      ($partition) = /\/(\w+).autoscr.dctl/;
      # Figure out what dc_shell command to use
      if (exists $partition_dcshcmds{$partition}) {
        # If there is a special one, use it
        $partition_dcshcmd = $partition_dcshcmds{$partition} 
      } else {
        # Otherwise, default
        $partition_dcshcmd = "dc_shell-t";
      }
      # Build the new command
      $new_dcshcmd = "$partition_dcshcmd $makefile_dcshcmd";
      if ($lsf) {
        $_ = "\tbsub -o \$(LOG_PATH)/${partition}.${bsublog} -K $bsubargs_all $partition_bsubargs{${partition}} \'$new_dcshcmd\'";
      } else {
        $_ = "\t$new_dcshcmd";
      }
    }
    print NEW;
  }
  close ORIG;
  close NEW;

  # Build make command
  $makecmd = "gmake" .
   " -j $joblimit" .
   " -f ${_acs_passdir}/Makefile";
  # Handle hotmods.  Hotmods are modules that should be given priority in
  # compiles.  In make, this is accomplished by listing them explicitly on
  # the command line, followed by "all".
  foreach $hotmod (@hotmods) {
    $makecmd .= " ${_acs_dest}/${hotmod}.db";
  }
  $makecmd .= " all";  # build everything else

  # Do the make
  unless ($nomake) {
    print "\n\nRunning make for pass $_acs_pass using command:\n\"$makecmd\"\n";
    open(MAKE,"$makecmd |");
    while(<MAKE>) {
      if (/Error:/) {
        print STDERR;
      }
      print;
    }
    close GMAKE;
    print "Done with make for pass $_acs_pass\n";
  }

  $_acs_prevpass = $_acs_pass
}

&msg("Exiting $me");

#####################################################################
#
# Subroutines
#
#####################################################################

# acsvars2env -- pass all _acs_ variables to shell ENV variables
# These can be accessed in all dctl code due to getenv call in .synopsys.
sub acsvars2env {

  for (keys %main:: ) {
    next unless ($_ =~ m/^_acs_/); # Only _acs_vars
  
    # Check to see the type of each variable - scalar, array or hash
  
    if (defined ${$main::{$_}}) {
      #print "Scalar variable $_ is ${$main::{$_}}\n";
      #$ENV{$_} = ${$main::{$_}};
      $ENV{$_} = $$_;
    }
    if (defined @{$main::{$_}}) {
      #print "Array variable $_ is @{$main::{$_}}\n";
      #$ENV{$_} = join(" ",@{$main::{$_}});
      $ENV{$_} = join(" ",@$_);
      
    }
    if (defined %{$main::{$_}}) {
      #print "Hash variable $_ is ",%{$main::{$_}},"\n";
      &hash2env("${_}__",%{$main::{$_}});
    }
  }

}

# rundc - run dc_shell
sub rundc {
  my($dcshcmd,$logfile,$bsublogfile,$lsf) = @_;
  &acsvars2env;
  if ($dcshcmd) {
    # respect it if already set (but eval to expand vars)
    $dcshcmd = eval("sprintf(\"%s\",\"$dcshcmd\")");
  } else {
    if ($_acs_mpc_top) {
      $dcshcmd = "psyn_shell -f ${_acs_scriptdir}/${scriptfile}"; 
    } else {
      $dcshcmd = "dc_shell-t -f ${_acs_scriptdir}/${scriptfile}"; 
    }
#    $dcshcmd .= " -x \'set search_path \"\$search_path $_acs_scriptdir\'";
  }
  if ($lsf) {
    # For lsf, create the command with the redirect to the logfile as part of
    # the command that bsub executes.  -o on bsub is used to log the lsf
    # output messages.
    # Unfortunately, there is no way to get bsub to send the output to stdout.
    # Therefore, we cannot use open(PASSLOG.. as below to watch the output.
    # It is possible to watch the output using tail, but you need a special
    # version of tail that always starts at the beginning of the file.  
    # Since this gets into OS and system specific features, I won't do this.
    $top_dcshcmd = "bsub -o ${bsublogfile} -K $bsubargs_all $partition_bsubargs{${_acs_top}} \'$dcshcmd > ${logfile}\'";
    print "\nRunning scriptfile \"$scriptfile\" using command \"$top_dcshcmd \"\n";
    system $top_dcshcmd;
  } else {
    # Without lsf, we can just execute the command and parse for error 
    # on the fly.
    print "\nRunning scriptfile \"$scriptfile\" using command \"$dcshcmd \"\n";
    open(PASSLOG,"$dcshcmd |tee ${logfile} |") || die "Couldn't open files to run $_acs_pass";
    while (<PASSLOG>) {
      if (/Error:/) {
        print STDERR;
      }
        #if ($verbose) {print};
        print;
    }
    close(PASSLOG);
  }
}

# makedir - make a directory
sub makedir {
  my ($dir) = @_;
  if (! -d "$dir") { 
    #mkdir("$dir",oct("777")) || die "Could not create directory $dir";
    system "mkdir -p $dir";
  } else {
    system "rm -fr $dir/*";
  }
}

# copy_files - copy a list of files to a dest directory
sub copy_files {
  my ($srcdir,$destdir,@files) = @_;
  foreach $file (@files) {
    if ($verbose) {
      print "Copying $srcdir/$file to $destdir/$file\n";
    }
    system "cp -pf $srcdir/$file $destdir/$file";
  }
}

sub hash2env {
  my ($prefix,%hash) = @_;

  foreach $key (keys %hash) {
    #print "key is $key\n";
    $ENV{"${prefix}${key}"} = $hash{$key};
  }

}

sub msg {
  my ($message) = @_;

  $date = `date`;
  chomp($date);
  $dateabs = `date +\%s`;
  chomp($dateabs);
  $hostname = `hostname`;
  chomp($hostname);

  print "-> $message   $date (abs:$dateabs) on system $hostname\n";
}

sub process_options {
  # Parse options
  use Getopt::Long;
  die unless GetOptions(
    "help" => \$help,
    "verbose" => \$verbose,
    "top=s" => \$_acs_top,
    "f=s" => \@morefiles,  # this is how chip data gets loaded
    "dcsh=s" => \$dcshcmd,
    "synlog=s" => \$synlog,
    "tclsuffix=s" => \$_acs_dctl,
    "hotmod=s" => \@hotmods,
    "globalsfile=s" => \@_acs_globals_files,  # global RTL vars to be read before code
    "libdir=s" => \@_acs_libdirs,  # library directories to be read after RTL
    "makeonly" => \$makeonly,  # just run make, not acs
    "nomake" => \$nomake,  # test mode - echo commands but don't execute
    "makefile=s" => \$makefile,
    "pass=s" => \@_acs_passes,  # pass(es) to run
    "joblimit=i" => \$joblimit,  # joblimit for make
    "scriptfile=s" => \$scriptfile, 
    "lsf" => \$lsf,
    "noacs" => \$noacs,
    "mpc_top" => \$_acs_mpc_top, 
    "scriptdir=s" => \$_acs_scriptdir, 
    "logdir=s" => \$_acs_logdir,
    );

  if ($help) {&show_help ; exit}

  # Check for required options
  if (!@morefiles) {
    print "Error: no -f specified.  Exiting\n";
    exit;
  }

  sub show_help {
   print STDERR "Syntax:
    $me [options] *.v

    options:
    -help                 Show this help
    -verbose              Be Chatty
    -top <module>         Top is \"module\"
    -f <file>             Read in file \"file\" (typically the .info.pl file)
    -scriptfile <scriptfile>   Use top level script file \"scriptfile\".
                          Defaults to $scriptfile.
    -dcsh <cmd>           Use \"cmd\" as the command to run dc_shell 
                          (defaults to \"dc_shell_t -f \${scriptfile}\")
                          To override, use \"\\\${scriptfile}\" as filename.
    -synlog <suffix>      Use \"suffix\" as the log file suffix
    -tclsuffix <suffix>   Use \"suffix\" as the tcl script file suffix
    -hotmod <module>      Module \"module\" should be given compile priority
                          (can be used multiple times)
    -globalsfile <file>   Add \"file\" to the list of files to be read ahead of
                          the HDL files (can be used multiple times)
    -libdir <dir>         Add \"dir\" to the list of library directories to be
                          searched after HDL files.
                          (can be used multiple times)
    -makeonly             Just run make.
    -nomake               Echo make command but don't execute
    -makefile <file>      Read makefile \"file\" to get list of verilog HDL files.
    -pass <passnumber>    Add \"passnumber\" to the list of passes to run.
                          (can be used multiple times)
                          If no -pass options given, will default to run pass 0,
                          pass 1, and pass 2.
    -j <joblimit>         Joblimit for gmake.
    -scriptdir <dir>      Directory for scripts (defaults to \".\")
    -logdir dir           Specify directory for synlog, bsublog files

    \n";
  }
}

# when doing require or use we must return 1
1

